@@ -12,25 +12,27 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _FSTPROCESSOR_
 #define _FSTPROCESSOR_
 
+#include <lttoolbox/ustring.h>
+#include <unicode/uchriter.h>
 #include <lttoolbox/alphabet.h>
 #include <lttoolbox/buffer.h>
-#include <lttoolbox/ltstr.h>
 #include <lttoolbox/my_stdio.h>
 #include <lttoolbox/state.h>
 #include <lttoolbox/trans_exe.h>
+#include <lttoolbox/input_file.h>
 #include <libxml/xmlreader.h>
 
-#include <cwchar>
 #include <map>
 #include <queue>
 #include <set>
 #include <string>
+#include <cstdint>
 
 using namespace std;
 
@@ -56,7 +58,7 @@ private:
   /**
    * Transducers in FSTP
    */
-  map<wstring, TransExe, Ltstr> transducers;
+  map<UString, TransExe> transducers;
 
   /**
    * Current state of lexical analysis
@@ -71,7 +73,7 @@ private:
   /**
    * Default value of weight unless specified
    */
-  double default_weight;
+  double default_weight = 0.0000;
 
   /**
    * The final states of inconditional sections in the dictionaries
@@ -101,27 +103,27 @@ private:
   /**
    * Queue of blanks, used in reading methods
    */
-  queue<wstring> blankqueue;
+  queue<UString> blankqueue;
 
   /**
    * Queue of wordbound blanks, used in reading methods
    */
-  queue<wstring> wblankqueue;
+  queue<UString> wblankqueue;
 
   /**
    * Set of characters being considered alphabetics
    */
-  set<wchar_t> alphabetic_chars;
+  set<UChar32> alphabetic_chars;
 
   /**
    * Set of characters to escape with a backslash
    */
-  set<wchar_t> escaped_chars;
+  set<UChar32> escaped_chars;
 
   /**
    * Set of characters to ignore
    */
-  set<wchar_t> ignored_chars;
+  set<UChar32> ignored_chars;
 
   /**
    * Mapping of characters for simplistic diacritic restoration specified in RCX files
@@ -141,7 +143,7 @@ private:
   /**
    * Input buffer
    */
-  Buffer<int> input_buffer;
+  Buffer<int32_t> input_buffer;
 
   /**
    * Begin of the transducer
@@ -151,86 +153,86 @@ private:
   /**
    * true if the position of input stream is out of a word
    */
-  bool outOfWord;
+  bool outOfWord = false;
 
   /**
    * true if we're automatically removing surface forms.
    */
-  bool biltransSurfaceForms;
+  bool biltransSurfaceForms = false;
 
 
   /**
    * if true, makes always difference between uppercase and lowercase
    * characters
    */
-  bool caseSensitive;
+  bool caseSensitive = false;
 
   /**
    * if true, uses the dictionary case, discarding surface case
    * information
    */
-  bool dictionaryCase;
+  bool dictionaryCase = false;
 
   /**
    * if true, flush the output when the null character is found
    */
-  bool nullFlush;
+  bool nullFlush = false;
 
   /**
    * nullFlush property for the skipUntil function
    */
-  bool nullFlushGeneration;
+  bool nullFlushGeneration = false;
 
   /**
    * if true, ignore the provided set of characters
    */
-  bool useIgnoredChars;
+  bool useIgnoredChars = false;
 
   /**
    * if true, attempt simplistic diacritic restoration
    */
-  bool useRestoreChars;
+  bool useRestoreChars = false;
 
   /**
    * if true, skips loading the default set of ignored characters
    */
-  bool useDefaultIgnoredChars;
+  bool useDefaultIgnoredChars = true;
 
   /**
    * if true, displays the final weights (if any)
    */
-  bool displayWeightsMode;
+  bool displayWeightsMode = false;
 
   /**
    * try analysing unknown words as compounds
    */
-  bool do_decomposition;
+  bool do_decomposition = false;
 
   /**
    * Symbol of CompoundOnlyL
    */
-  int compoundOnlyLSymbol;
+  int compoundOnlyLSymbol = 0;
 
   /**
    * Symbol of CompoundR
    */
-  int compoundRSymbol;
+  int compoundRSymbol = 0;
 
   /**
    * Show or not the controls symbols (as compoundRSymbol)
    */
-   bool showControlSymbols;
+  bool showControlSymbols = false;
 
   /**
    * Max compound elements
    * Hard coded for now, but there might come a switch one day
    */
-  int compound_max_elements;
+  int compound_max_elements = 4;
 
   /**
    * Output no more than 'N' number of weighted analyses
    */
-  int maxAnalyses;
+  int maxAnalyses = INT_MAX;
 
   /**
    * True if a wblank block ([[..]]xyz[[/]]) was just read
@@ -250,62 +252,41 @@ private:
   /**
    * Output no more than 'N' best weight classes
    */
-  int maxWeightClasses;
+  int maxWeightClasses = INT_MAX;
 
   /**
    * Prints an error of input stream and exits
    */
   void streamError();
 
-  /**
-   * Reads a character that is defined in the set of escaped_chars
-   * @param input the stream to read from
-   * @return code of the character
-   */
-  wchar_t readEscaped(FILE *input);
-
-  /**
-   * Reads a block from the stream input, enclosed by delim1 and delim2
-   * @param input the stream being read
-   * @param delim1 the delimiter of the beginning of the sequence
-   * @param delim1 the delimiter of the end of the sequence
-   */
-  wstring readFullBlock(FILE *input, wchar_t const delim1, wchar_t const delim2);
-
-  /**
-   * Reads a wordbound blank from the stream input
-   * @param input the stream being read
-   */
-  wstring readWblank(FILE *input);
-
   /**
    * Reads a wordbound blank (opening blank to closing blank) from the stream input -> [[...]]xyz[[/]]
    * @param input the stream being read
    * @param output the stream to write on
    * @return true if the word enclosed by the wordbound blank has a ~ for postgeneration activation
    */
-  bool wblankPostGen(FILE *input, FILE *output);
+  bool wblankPostGen(InputFile& input, UFILE *output);
 
   /**
    * Returns true if the character code is identified as alphabetic
    * @param c the code provided by the user
    * @return true if it's alphabetic
    */
-  bool isAlphabetic(wchar_t const c) const;
+  bool isAlphabetic(UChar32 const c) const;
 
   /**
    * Tests if a character is in the set of escaped_chars
    * @param c the character code provided by the user
    * @return true if it is in the set
    */
-  bool isEscaped(wchar_t const c) const;
+  bool isEscaped(UChar32 const c) const;
 
   /**
    * Read text from stream (analysis version)
    * @param input the stream to read
    * @return the next symbol in the stream
    */
-  int readAnalysis(FILE *input);
+  int readAnalysis(InputFile& input);
 
   /**
    * Read text from stream (decomposition version)
@@ -313,7 +294,7 @@ private:
    * @param output the stream to write on
    * @return the next symbol in the stream
    */
-  int readDecomposition(FILE *input, FILE *output);
+  int readDecomposition(InputFile& input, UFILE *output);
 
   /**
    * Read text from stream (postgeneration version)
@@ -321,7 +302,7 @@ private:
    * @param output the stream to write on
    * @return the next symbol in the stream
    */
-  int readPostgeneration(FILE *input, FILE *output);
+  int readPostgeneration(InputFile& input, UFILE *output);
 
   /**
    * Read text from stream (generation version)
@@ -329,7 +310,7 @@ private:
    * @param output the stream being written to
    * @return the next symbol in the stream
    */
-  int readGeneration(FILE *input, FILE *output);
+  int readGeneration(InputFile& input, UFILE *output);
 
   /**
    * Read text from stream (biltrans version)
@@ -337,32 +318,40 @@ private:
    * @param output the stream to write on
    * @return the queue of 0-symbols, and the next symbol in the stream
    */
-  pair<wstring, int> readBilingual(FILE *input, FILE *output);
+  pair<UString, int> readBilingual(InputFile& input, UFILE *output);
 
   /**
    * Read text from stream (SAO version)
    * @param input the stream to read
    * @return the next symbol in the stream
    */
-  int readSAO(FILE *input);
+  int readSAO(InputFile& input);
 
   /**
    * Flush all the blanks remaining in the current process
    * @param output stream to write blanks
    */
-  void flushBlanks(FILE *output);
+  void flushBlanks(UFILE *output);
 
   /**
    * Flush all the wordbound blanks remaining in the current process
    * @param output stream to write blanks
    */
-  void flushWblanks(FILE *output);
+  void flushWblanks(UFILE *output);
 
   /**
-   * Combine wordbound blanks in the queue and return them
+   * Combine wordbound blanks in the queue and return them.
+   *
+   * May pop from 'wblankqueue' and set 'need_end_wblank' to true.
+   *
+   * If 'wblankqueue' (see which) is empty, we get an empty string,
+   * otherwise we return a semicolon-separated combination of opening
+   * wblanks in the queue. If there is only a closing wblank, we just
+   * set need_end_wblank.
+   *
    * @return final wblank string
   */
-  wstring combineWblanks();
+  UString combineWblanks();
 
   /**
    * Calculate the initial state of parsing
@@ -379,7 +368,7 @@ private:
    * @param str the string to write, escaping characters
    * @param output the stream to write in
    */
-  void writeEscaped(wstring const &str, FILE *output);
+  void writeEscaped(UString const &str, UFILE *output);
 
   /**
    * Write a string to an output stream.
@@ -390,7 +379,7 @@ private:
    * @param output the stream to write in
    * @return how many blanks to pop and print after printing lu
    */
-  size_t writeEscapedPopBlanks(wstring const &str, FILE *output);
+  size_t writeEscapedPopBlanks(UString const &str, UFILE *output);
 
   /**
    * Write a string to an output stream, escaping all escapable characters
@@ -398,7 +387,7 @@ private:
    * @param str the string to write, escaping characters
    * @param output the stream to write in
    */
-  void writeEscapedWithTags(wstring const &str, FILE *output);
+  void writeEscapedWithTags(UString const &str, UFILE *output);
 
 
   /**
@@ -407,7 +396,7 @@ private:
    * @param the searched suffix
    * @returns true if 'str' has the suffix 'suffix'
    */
-  static bool endsWith(wstring const &str, wstring const &suffix);
+  static bool endsWith(UString const &str, UString const &suffix);
 
   /**
    * Prints a word
@@ -415,7 +404,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWord(wstring const &sf, wstring const &lf, FILE *output);
+  void printWord(UString const &sf, UString const &lf, UFILE *output);
 
   /**
    * Prints a word.
@@ -425,7 +414,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWordPopBlank(wstring const &sf, wstring const &lf, FILE *output);
+  void printWordPopBlank(UString const &sf, UString const &lf, UFILE *output);
 
   /**
    * Prints a word (Bilingual version)
@@ -433,7 +422,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWordBilingual(wstring const &sf, wstring const &lf, FILE *output);
+  void printWordBilingual(UString const &sf, UString const &lf, UFILE *output);
 
 
   /**
@@ -441,21 +430,21 @@ private:
    * @param lf lexical form
    * @param output stream where the word is written
    */
-  void printSAOWord(wstring const &lf, FILE *output);
+  void printSAOWord(UString const &lf, UFILE *output);
 
   /**
    * Prints an unknown word
    * @param sf surface form of the word
    * @param output stream where the word is written
    */
-  void printUnknownWord(wstring const &sf, FILE *output);
+  void printUnknownWord(UString const &sf, UFILE *output);
 
   void initDecompositionSymbols();
 
-  vector<wstring> numbers;
-  int readTMAnalysis(FILE *input);
+  vector<UString> numbers;
+  int readTMAnalysis(InputFile& input);
 
-  unsigned int lastBlank(wstring const &str);
+  unsigned int lastBlank(UString const &str);
 
   /**
    * Print one blankqueue item if there is one, or a given "space" value.
@@ -463,32 +452,46 @@ private:
    * @param val the space character to use if no blank queue
    * @param output stream where the word is written
    */
-  void printSpace(wchar_t const val, FILE *output);
+  void printSpace(UChar32 const val, UFILE *output);
 
-  void skipUntil(FILE *input, FILE *output, wint_t const character);
-  static wstring removeTags(wstring const &str);
-  wstring compoundAnalysis(wstring str, bool uppercase, bool firstupper);
-  size_t firstNotAlpha(wstring const &sf);
+  void skipUntil(InputFile& input, UFILE *output, UChar32 const character);
+  static UString removeTags(UString const &str);
+  UString compoundAnalysis(UString str, bool uppercase, bool firstupper);
+  size_t firstNotAlpha(UString const &sf);
 
-  void analysis_wrapper_null_flush(FILE *input, FILE *output);
-  void lsx_wrapper_null_flush(FILE *input, FILE *output);
-  void bilingual_wrapper_null_flush(FILE *input, FILE *output);
-  void generation_wrapper_null_flush(FILE *input, FILE *output,
+  void analysis_wrapper_null_flush(InputFile& input, UFILE *output);
+  void bilingual_wrapper_null_flush(InputFile& input, UFILE *output, GenerationMode mode = gm_unknown);
+  void generation_wrapper_null_flush(InputFile& input, UFILE *output,
                                      GenerationMode mode);
-  void postgeneration_wrapper_null_flush(FILE *input, FILE *output);
-  void intergeneration_wrapper_null_flush(FILE *input, FILE *output);
-  void transliteration_wrapper_null_flush(FILE *input, FILE *output);
+  void postgeneration_wrapper_null_flush(InputFile& input, UFILE *output);
+  void intergeneration_wrapper_null_flush(InputFile& input, UFILE *output);
+  void transliteration_wrapper_null_flush(InputFile& input, UFILE *output);
 
-  wstring compose(wstring const &lexforms, wstring const &queue) const;
+  UString compose(UString const &lexforms, UString const &queue) const;
 
   void procNodeICX();
   void procNodeRCX();
   void initDefaultIgnoredCharacters();
 
-  bool isLastBlankTM;
+  bool isLastBlankTM = false;
 
   xmlTextReaderPtr reader;
 public:
+
+  /*
+   * String constants
+   */
+  static UString const XML_TEXT_NODE;
+  static UString const XML_COMMENT_NODE;
+  static UString const XML_IGNORED_CHARS_ELEM;
+  static UString const XML_RESTORE_CHAR_ELEM;
+  static UString const XML_RESTORE_CHARS_ELEM;
+  static UString const XML_VALUE_ATTR;
+  static UString const XML_CHAR_ELEM;
+  static UString const WBLANK_START;
+  static UString const WBLANK_END;
+  static UString const WBLANK_FINAL;
+
   FSTProcessor();
 
   void initAnalysis();
@@ -499,25 +502,23 @@ public:
   void initBiltrans();
   void initDecomposition();
 
-  void analysis(FILE *input = stdin, FILE *output = stdout);
-  void tm_analysis(FILE *input = stdin, FILE *output = stdout);
-  void generation(FILE *input = stdin, FILE *output = stdout, GenerationMode mode = gm_unknown);
-  void postgeneration(FILE *input = stdin, FILE *output = stdout);
-  void intergeneration(FILE *input = stdin, FILE *output = stdout);
-  void transliteration(FILE *input = stdin, FILE *output = stdout);
-  wstring biltrans(wstring const &input_word, bool with_delim = true);
-  wstring biltransfull(wstring const &input_word, bool with_delim = true);
-  void bilingual(FILE *input = stdin, FILE *output = stdout);
-  pair<wstring, int> biltransWithQueue(wstring const &input_word, bool with_delim = true);
-  wstring biltransWithoutQueue(wstring const &input_word, bool with_delim = true);
-  void SAO(FILE *input = stdin, FILE *output = stdout);
+  void analysis(InputFile& input, UFILE *output);
+  void tm_analysis(InputFile& input, UFILE *output);
+  void generation(InputFile& input, UFILE *output, GenerationMode mode = gm_unknown);
+  void postgeneration(InputFile& input, UFILE *output);
+  void intergeneration(InputFile& input, UFILE *output);
+  void transliteration(InputFile& input, UFILE *output);
+  UString biltrans(UString const &input_word, bool with_delim = true);
+  UString biltransfull(UString const &input_word, bool with_delim = true);
+  void bilingual(InputFile& input, UFILE *output, GenerationMode mode = gm_unknown);
+  pair<UString, int> biltransWithQueue(UString const &input_word, bool with_delim = true);
+  UString biltransWithoutQueue(UString const &input_word, bool with_delim = true);
+  void SAO(InputFile& input, UFILE *output);
   void parseICX(string const &file);
   void parseRCX(string const &file);
 
   void load(FILE *input);
 
-  void lsx(FILE *input, FILE *output);
-
   bool valid() const;
 
   void setCaseSensitiveMode(bool const value);
