@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <lttoolbox/fst_processor.h>
 #include <lttoolbox/my_stdio.h>
@@ -28,9 +28,6 @@
 #include <fcntl.h>
 #endif
 
-#if defined(_WIN32) && !defined(_MSC_VER)
-#include <utf8_fwrap.h>
-#endif
 
 using namespace std;
 
@@ -104,6 +101,8 @@ void checkValidity(FSTProcessor const &fstp)
 
 int main(int argc, char *argv[])
 {
+  LtLocale::tryToSetLocale();
+
   int cmd = 0;
   int maxAnalyses;
   int maxWeightClasses;
@@ -139,6 +138,7 @@ int main(int argc, char *argv[])
     };
 #endif
 
+  GenerationMode bilmode = gm_unknown;
   while(true)
   {
 #if HAVE_GETOPT_LONG
@@ -182,7 +182,7 @@ int main(int argc, char *argv[])
       maxAnalyses = atoi(optarg);
       if (maxAnalyses < 1)
       {
-        wcerr << "Invalid or no argument for analyses count" << endl;
+        cerr << "Invalid or no argument for analyses count" << endl;
         exit(EXIT_FAILURE);
       }
       fstp.setMaxAnalysesValue(maxAnalyses);
@@ -192,7 +192,7 @@ int main(int argc, char *argv[])
       maxWeightClasses = atoi(optarg);
       if (maxWeightClasses < 1)
       {
-        wcerr << "Invalid or no argument for weight class count" << endl;
+        cerr << "Invalid or no argument for weight class count" << endl;
         exit(EXIT_FAILURE);
       }
       fstp.setMaxWeightClassesValue(maxWeightClasses);
@@ -216,6 +216,16 @@ int main(int argc, char *argv[])
       {
         cmd = c;
       }
+      else if(cmd == 'g' && c == 'b') {
+        // "lt-proc -g -b generador.bin" should run biltrans, keeping unknown-marks
+        bilmode = gm_unknown;
+        cmd = 'b';
+      }
+      else if(cmd == 'n' && c == 'b') {
+        // "lt-proc -n -b generador.bin" should run biltrans, stripping unknown-marks
+        bilmode = gm_clean;
+        cmd = 'b';
+      }
       else
       {
         endProgram(argv[0]);
@@ -241,29 +251,27 @@ int main(int argc, char *argv[])
     }
   }
 
-  FILE *input = stdin, *output = stdout;
-  LtLocale::tryToSetLocale();
+  InputFile input;
+  UFILE* output = u_finit(stdout, NULL, NULL);
 
   if(optind == (argc - 3))
   {
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      wcerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
+      cerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
-    input = fopen(argv[optind+1], "rb");
-    if(input == NULL || ferror(input))
-    {
-      wcerr << "Error: Cannot open file '" << argv[optind+1] << "'." << endl << endl;
+    if (!input.open(argv[optind+1])) {
+      cerr << "Error: Cannot open file '" << argv[optind+1] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
-    output= fopen(argv[optind+2], "wb");
-    if(output == NULL || ferror(output))
+    output = u_fopen(argv[optind+2], "wb", NULL, NULL);
+    if(output == NULL)
     {
-      wcerr << "Error: Cannot open file '" << argv[optind+2] << "'." << endl << endl;
+      cerr << "Error: Cannot open file '" << argv[optind+2] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
@@ -275,14 +283,12 @@ int main(int argc, char *argv[])
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      wcerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
+      cerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
-    input = fopen(argv[optind+1], "rb");
-    if(input == NULL || ferror(input))
-    {
-      wcerr << "Error: Cannot open file '" << argv[optind+1] << "'." << endl << endl;
+    if (!input.open(argv[optind+1])) {
+      cerr << "Error: Cannot open file '" << argv[optind+1] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
@@ -294,7 +300,7 @@ int main(int argc, char *argv[])
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      wcerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
+      cerr << "Error: Cannot open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
      }
     fstp.load(in);
@@ -378,13 +384,13 @@ int main(int argc, char *argv[])
         fstp.initBiltrans();
         checkValidity(fstp);
         fstp.setBiltransSurfaceForms(true);
-        fstp.bilingual(input, output);
+        fstp.bilingual(input, output, bilmode);
         break;
 
       case 'b':
         fstp.initBiltrans();
         checkValidity(fstp);
-        fstp.bilingual(input, output);
+        fstp.bilingual(input, output, bilmode);
         break;
 
       case 'e':
@@ -403,15 +409,14 @@ int main(int argc, char *argv[])
   }
   catch (exception& e)
   {
-    wcerr << e.what();
+    cerr << e.what();
     if (fstp.getNullFlush()) {
-      fputwc_unlocked(L'\0', output);
+      u_fputc('\0', output);
     }
 
     exit(1);
   }
 
-  fclose(input);
-  fclose(output);
+  u_fclose(output);
   return EXIT_SUCCESS;
 }
