@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _PATTERNLIST_
 #define _PATTERNLIST_
@@ -34,55 +34,56 @@ typedef pair<PatternStore::iterator, PatternStore::iterator> PatternRange;
 class PatternList
 {
 private:
-  Alphabet alphabet;  
+  Alphabet alphabet;
   PatternStore patterns;
   bool sequence;
   list<vector<int> > sequence_data;
   Transducer transducer;
   map<int, int> final_type;
   int sequence_id;
+  double default_weight;
 
   void copy(PatternList const &o);
   void destroy();
-  void insertOutOfSequence(wstring const &lemma, wstring const &tags,
-			   vector<int> &result);
-  void insertIntoSequence(int const id, wstring const &lemma, 
-			  wstring const &tags);
+  void insertOutOfSequence(UString const &lemma, UString const &tags,
+                           vector<int> &result);
+  void insertIntoSequence(int const id, UString const &lemma,
+                          UString const &tags);
+
+  static int tagCount(UString const &tags);
+  static UString tagAt(UString const &tags, int const index);
 
-  static int tagCount(wstring const &tags);
-  static wstring tagAt(wstring const &tags, int const index);
-  
 public:
   /**
    * This symbol stands for any char
    */
-  static wstring const ANY_CHAR;
+  static UString const ANY_CHAR;
 
   /**
    * This symbol stands for any tag
    */
-  static wstring const ANY_TAG;
+  static UString const ANY_TAG;
 
   /**
    * This symbol marks a word queue
    */
-  static wstring const QUEUE;
+  static UString const QUEUE;
 
   /**
    * Constructor
    */
   PatternList();
-  
+
   /**
    * Destructor
    */
   ~PatternList();
-  
+
   /**
    * Copy constructor
    */
   PatternList(PatternList const &o);
-  
+
   /**
    * Assignment operator
    * @param o the object to be assigned
@@ -93,19 +94,19 @@ public:
    * Marks the start of a multiple word sequence
    */
   void beginSequence();
-  
+
   /**
-   * Ends the multiple word sequence 
+   * Ends the multiple word sequence
    */
   void endSequence();
-  
+
   /**
    * Insertion method
    * @param id
    * @param lemma
    * @param tags
    */
-  void insert(int const id, wstring const &lemma, wstring const &tags);
+  void insert(int const id, UString const &lemma, UString const &tags);
 
   /**
    * Insertion method
@@ -113,24 +114,24 @@ public:
    * @param otherid
    */
   void insert(int const id, int const otherid);
-  
+
   /**
    * Get the PatternStore
    * @returns a PatternStore object
    */
   PatternStore const & getPatterns();
-  
+
   /**
    * Build the FSM
    */
   void buildTransducer();
-  
+
   /**
    * Read PatternList from a file
    * @param input the input stream
    */
   void read(FILE *input);
-  
+
   /**
    * Write PatternList to a file
    * @param output the output stream
@@ -139,13 +140,13 @@ public:
 
   void serialise(std::ostream &serialised) const;
   void deserialise(std::istream &serialised);
-  
+
   /**
    * Create a new MatchExe from PatternList, must be freed with 'delete'
    * @return the new MatchExe object
    */
   MatchExe * newMatchExe() const;
-  
+
   /**
    * Get the alphabet of this PatternList object
    * @return the alphabet
